﻿-- Written by : Rakreo of Duskwood

-- Capture the current version of the addon from the .toc file
local CURRENT_VERSION = GetAddOnMetadata('SnackShop', 'Version');

-- Build an array of Achievements to track
local Achievements = {
	[6026] = true, -- World Events / Darkmoon Faire / Fairegoer's Feast
	[5753] = true, -- General Cataclysmically Delicious
	[5754] = true, -- General / Drown Your Sorrows
	[7329] = true, -- General / Pandaren Cuisine
	[7330] = true, -- General / Pandaren Delicacies
	[9502] = true, -- General / Draenor Cuisine
	[1780] = true, -- Professions / Cooking / Sencond That Emotion
};

-- Variables used by this addon
local realm, faction, player;

-- Build translation table L
local locale, L = GetLocale(), {};
-- Translations for the French client
if (locale == 'frFR') then
	L['consume'] = 'Besoin de consommer';
-- Translations for the German client
elseif (locale == 'deDE') then
	L['consume'] = 'Verbrauchen müssen';
-- Translations for the Korean client
elseif (locale == 'koKR') then
	L['consume'] = '소비 할 필요';
-- Translations for the Chinese (simplified) client
elseif (locale == 'zhCN') then
	L['consume'] = '需要消耗';
-- Translations for the Chinese (traditional) client
elseif (locale == 'zhTW') then
	L['consume'] = '需要消耗';
-- Translations for the Russian client
elseif (locale == 'ruRU') then
	L['consume'] = 'Нужно потреблять';
-- Translations for the Spanish (Spain) client
elseif (locale == 'esES') then
	L['consume'] = 'Necesidad de consumir';
-- Translations for the Spanish (Mexico) client
elseif (locale == 'esMX') then
	L['consume'] = 'Necesidad de consumir';
-- Translations for the English client
else
	L['consume'] = 'Need to consume';
end;

----------------------------------------------------------
-- Create the Frame and Register all the Events we need --
----------------------------------------------------------
-- Create a Frame for this addon so we can attach scripts and define the array for the functions that need to be registered
local SnackShop, events = CreateFrame('Frame'), {};
---------------------------------------------------------------------
-- Fired after the Saved Variables for this addon have been loaded --
---------------------------------------------------------------------
function events:ADDON_LOADED(...)
 -- Fired after the Saved Variables for this addon have been loaded
	if ... == 'SnackShop' then
		-- Make sure the database has been populated with the defaults
		SnackShop:InitializeDB();
	end;
end;
-----------------------------------------------------------------------------------------------------------------------------
-- Fired when the games first loads or when entering/leaving an instance (Basically anytime the 'Loading' screen is shown) --
-----------------------------------------------------------------------------------------------------------------------------
function events:PLAYER_ENTERING_WORLD(...)
	-- Scan the current players Achievements and store them in the DB
	SnackShop:ScanAchievements();
end;
-------------------------------------------------------------
-- Fired when the criteria for an achievement has changed. --
-------------------------------------------------------------
function events:CRITERIA_UPDATE(...)
	-- Scan the current players Achievements and store them in the DB
	SnackShop:ScanAchievements();
end;
------------------------------------
-- Fired when a player levels up. --
------------------------------------
function events:PLAYER_LEVEL_UP(...)
	-- Capture the new level of the player
	local level = ...;
	-- Make sure the DataBase has a default ['Realm'][realm][player]['level']
	SnackShopDB['Realm'][realm][player]['level'] = level;
end;
------------------------------------------------------------------------------------------------------------
-- Set the 'OnEvent' trigger assigned to SnackShop and execute the function based on the Event that fired --
------------------------------------------------------------------------------------------------------------
SnackShop:SetScript('OnEvent', function(self, event, ...) events[event](self, ...); end);
-- Loop thru all the 'events' array and register the 'Events'
for k, v in pairs(events) do
	-- Register all events for which handlers have been defined
	SnackShop:RegisterEvent(k);
end;
-----------------------------------------------------------
-- Function to make sure the DataBase has default values --
-----------------------------------------------------------
function SnackShop:InitializeDB()
	-- Capture the current 'Realm', 'Faction' and 'Player'
	realm, faction, player = GetRealmName(), UnitFactionGroup('player'), UnitName('player');
	-- If there is not SnackShopDB then set it to a blank array
	if not SnackShopDB then SnackShopDB = {}; end;
	-- Make sure the DataBase version gets updated
	SnackShopDB['Version'] = CURRENT_VERSION;
	-- Make sure the DataBase has a default ['Realm']
	SnackShopDB['Realm'] = SnackShopDB['Realm'] or {};
	-- Make sure the DataBase has a default ['Realm'][realm]
	SnackShopDB['Realm'][realm] = SnackShopDB['Realm'][realm] or {};
	-- Make sure the DataBase has a default ['Realm'][realm][player]
	SnackShopDB['Realm'][realm][player] = SnackShopDB['Realm'][realm][player] or {};
	-- Make sure the DataBase has a default ['Realm'][realm][player]['faction']
	SnackShopDB['Realm'][realm][player]['faction'] = faction;
	-- Make sure the DataBase has a default ['Realm'][realm][player]['level']
	SnackShopDB['Realm'][realm][player]['level'] = UnitLevel("player");
	-- Make sure the DataBase has a default ['Realm'][realm][player]['criteria']
	SnackShopDB['Realm'][realm][player]['criteria'] = SnackShopDB['Realm'][realm][player]['criteria'] or {};
end;
---------------------------------------------------------------------------------------
-- Function to scan the players current Achievements and store the results in the DB --
---------------------------------------------------------------------------------------
function SnackShop:ScanAchievements()
	-- Loop thru each Achievement and see what we have completed on this toon
	for achievementID, _ in pairs(Achievements) do
		-- Loop thru each criteria andsee if it is completed
		for i = 1, GetAchievementNumCriteria(achievementID) do
			-- Get the status of this criteria
			local criteriaString, _, completed = GetAchievementCriteriaInfo(achievementID, i);
			-- If this toon has completed this criteria then stop tracking it
			if completed == true then SnackShopDB['Realm'][realm][player]['criteria'][criteriaString] = nil
			-- We must not have completed this criteria so track it
			else SnackShopDB['Realm'][realm][player]['criteria'][criteriaString] = true end;
		end;
	end;
end;
-------------------------------------------------------
-- Create all the Hooks into the appropiate Tooltips --
-------------------------------------------------------
local tips = {GameTooltip, ItemRefTooltip};
-- Define the Methods are are going to Hook into
local method = {'OnTooltipSetItem'};
-- Loop thru each Tooltip Type
for _, tip in pairs(tips) do
	-- Loop thru each Tooltip Method
	for _, hook in pairs(method) do
		-- Create the actual Hook into the Tooltip
		tip:HookScript(hook, function(tooltip, ...)
			-- Make an array to hold the toons who need this item
			local needed, toon, item, txt = {}, nil, nil, nil;
			-- Capture the Name for the current Item in the Tooltip
			local itemName, itemLink = tooltip:GetItem();
			--Make sure we have an itemLink
			if itemLink then
				-- Extract the ItemID from the Link and make sure we change it to a number
				local itemID = tonumber(itemLink:match('item:(%d+):'));
				-- Get the Required Level of the Item
				local _, _, _, _, reqLevel = GetItemInfo(itemID);
				-- Make sure we have a valid Item name
				if itemName then
					-- Loop thru all the toons on this Realm and see who needs it
					for toon, _ in pairs(SnackShopDB['Realm'][realm]) do
						-- Loop thru all the items needed by this toon
						for item, _ in pairs(SnackShopDB['Realm'][realm][toon]['criteria']) do
							-- If this item is needed by this toon then add the toon to the needed array
							if item == itemName then
								-- Color code the toon's name based on if the toon can currently consume the item
								if SnackShopDB['Realm'][realm][toon]['level'] >= reqLevel then
									-- See if we are going to show ourself
									if toon == player then
										txt = '|cff6666ff'..toon..'|r';
									else
										txt = '|cff00ff00'..toon..'|r';
									end;
								else
									if toon == player then
										txt = '|cffff6699'..toon..' ('..SnackShopDB['Realm'][realm][toon]['level']..')|r';
									else
										txt = '|cffff0000'..toon..' ('..SnackShopDB['Realm'][realm][toon]['level']..')|r';
									end;
								end;
								-- Remember the toons name for later
								table.insert(needed, txt)
							end;
						end;
					end;
				end;
			end;
			-- If we have any toons we need to show
			if #needed > 0 then
				-- Sort table in alphabetical order
				table.sort(needed);
				-- Insert this line into the Tooltip
				tooltip:AddLine('\n'..L['consume']..': '..table.concat(needed, ', ')..'|r\n', 0.9, 0.9, 0.6, 1)
				end;
				-- Make sure to 'Repaint' the Tooltip to show the newly added lines
			tooltip:Show();
		end)
	end;
end;